--[[

Broker_Reagents, a reagent tracker for World of Warcraft.
Copyright (C) 2009 Rabbit

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

]]

local reagentSets = {
	DEATHKNIGHT = {
		-- Corpse Dust
		37201,
	},
	MAGE = {
		-- Arcane Powder
		17020,
		-- Rune of Teleportation
		17031,
		-- Rune of Portals
		17032,
		-- Light Feather
		17056,
	},
	PALADIN = {
		-- Symbol of Kings
		21177,
		-- Symbol of Divinity
		17033,
	},
	PRIEST = {
		-- Light Feather
		17056,
	},
	SHAMAN = {
		-- Ankh
		17030,
		-- Shiny Fish Scales
		17057,
		-- Fish Oil
		17058,
	},
	WARLOCK = {
		-- Infernal Stone
		5565,
		-- Soul Shard
		6265,
		-- Demonic Figurine
		16583,
	},
	ROGUE = {
		-- Thistle Tea
		7676,
	},
}

local collapsingSets = {
	WARLOCK = {
		-- Firestone
		{ 41170, 41169, 41171, 41172, 40773, 41173, 41174 },
		-- Healthstone
		{ 19004, 19005, 5512,
		  19007, 19006, 5511,
		  19009, 5509, 19008,
		  5510, 19011, 19010,
		  19013, 9421, 19012,
		  22105, 22103, 22104,
		  36889, 36890, 36891,
		  36894, 36893, 36892 },
		-- Soulstone
		{ 5232, 16892, 16893, 16895, 16896, 22116, 36895 },
		-- Spellstone
		{ 41191, 41192, 41193, 41194, 41195, 41196 },
	},
	ROGUE = {
		-- Anesthetic
		{ 21835, 43237 },
		-- Wound
		{ 10918, 10920, 10921, 10922, 22055, 43234, 43235 },
		-- Mind Numbing
		{ 5237 },
		-- Instant
		{ 6947, 6949, 6950, 8926, 8927, 8928, 21927, 43230, 43231 },
		-- Crippling
		{ 3775 },
		-- Deadly
		{ 2892, 2893, 8984, 8985, 20844, 22053, 22054, 43232, 43233 },
	},
	MAGE = {
		-- Mana Gem
		{ 5514, 5513, 8007, 8008, 22044, 33312 },
	},
	DRUID = {
		-- Gift of the Wild
		{ 17021, 17026, 22148, 44605 },
		-- Rebirth
		{ 17034, 17035, 17036, 17037, 17038, 22147, 44614 },
	},
	PRIEST = {
		-- Candles
		{ 17028, 17029, 44615 },
	},
}

local healingPotions = {
	929, 118, 858, 2456, 4596, 1710, 9144, 3928, 13446, 28100, 18253, 22829, 33092, 22850, 39671, 34440, 33447, 40087, 43569, 40077
}

local manaPotions = {
	3827, 2456, 2455, 3385, 6149, 9144, 13443, 13444, 18253, 28101, 22832, 33093, 31677, 22850, 34440, 40067, 33448, 40087, 43570, 40077
}

--[[
function BRTEST()
	for k, v in pairs(reagentSets) do
		for i, reagent in next, v do
			local name = GetItemInfo(reagent)
			if not name then print("No return from: " .. tostring(reagent)) end
		end
	end
	for k, v in pairs(collapsingSets) do
		for i, set in next, v do
			for z, item in next, set do
				local name = GetItemInfo(item)
				if not name then print("No return from: " .. tostring(item)) end
			end
		end
	end
	for i, v in next, healingPotions do
		local name = GetItemInfo(v)
		if not name then print("No return from: " .. tostring(v)) end
	end
	for i, v in next, manaPotions do
		local name = GetItemInfo(v)
		if not name then print("No return from: " .. tostring(v)) end
	end
	print("done")
end]]

local track = nil
local trackAsOne = nil

local reagentCount = {}
local reagentNames = {}
local max = {}
local icons = {}
local colors = {}
local db = nil

local classIcons = {
	DRUID = "Interface\\Icons\\INV_Misc_Branch_01",
	MAGE = "Interface\\Icons\\INV_Misc_Dust_01",
	PALADIN = "Interface\\Icons\\INV_Stone_WeightStone_05",
	PRIEST = "Interface\\Icons\\INV_Misc_Candle_03",
	ROGUE = "Interface\\Icons\\Trade_BrewPoison",
	SHAMAN = "Interface\\Icons\\INV_Jewelry_Talisman_06",
	WARLOCK = "Interface\\Icons\\INV_Misc_Gem_Amethyst_02",
	DEATHKNIGHT = "Interface\\Icons\\INV_Misc_Dust_02",
	HUNTER = "Interface\\Icons\\INV_Ammo_Arrow_02",
	WARRIOR = "Interface\\Icons\\Misc_Book_09",
}

local addon = CreateFrame("Frame")

local qualityColor = {}

local menu = {}
local menuItems = {}
local popupFrame = CreateFrame("Frame", "BrokerReagentsMenu", addon, "UIDropDownMenuTemplate")
local function openMenu(frame) EasyMenu(menu, popupFrame, frame, 20, 4, "MENU") end
local _, playerClass = UnitClass("player")
local ldb = LibStub("LibDataBroker-1.1")
local bReagents = ldb:NewDataObject("Reagents", {
	type = "data source",
	icon = classIcons[playerClass],
	text = "Reagents",
	OnClick = openMenu,
})
if not bReagents then return end

local needsUpdate = true -- Force update on login, even though we know that BAG_UPDATE fires on login

local function updateText()
	local count = 0
	for i, v in next, reagentNames do
		if not db[v] and reagentCount[v] < max[v] then
			count = count + 1
		end
	end
	bReagents.text = count > 0 and "|cffff0000" .. count .. "|r Reagents" or "Reagents"
end

local function update()
	wipe(reagentNames)

	for i, v in next, track do
		local name, link, rarity, _, _, _, _, m, _, icon = GetItemInfo(v)
		if name then
			reagentNames[#reagentNames + 1] = name
			reagentCount[name] = GetItemCount(link)
			max[name] = m
			icons[name] = icon
			colors[name] = qualityColor[rarity + 1]
		end
	end

	for i, set in next, trackAsOne do
		local counter = 0
		local stackSize = 0
		local setIcon = nil
		local setRarity = 0
		local properName = nil
		for z, item in next, set do
			local name, link, rarity, _, _, _, _, m, _, icon = GetItemInfo(item)
			if name then
				if m > stackSize then stackSize = m end
				if rarity > setRarity then setRarity = rarity end
				if not properName then properName = name end
				if not setIcon then setIcon = icon end
				counter = counter + GetItemCount(item)
			end
		end
		if properName then
			reagentNames[#reagentNames + 1] = properName
			reagentCount[properName] = counter
			max[properName] = stackSize
			icons[properName] = setIcon
			colors[properName] = qualityColor[setRarity + 1]
		end
	end

	table.sort(reagentNames)

	wipe(menu)
	for i, v in next, reagentNames do
		local item = menuItems[v]
		if not item then
			menuItems[v] = {
				text = v,
				tooltipTitle = v,
				tooltipText = ("Toggle whether or not the addon should track %s."):format(v),
				func = function() db[v] = not db[v]; updateText() end,
				checked = function() return not db[v] end,
			}
			item = menuItems[v]
		end
		menu[#menu + 1] = item
	end
	
	updateText()

	needsUpdate = nil
end
function addon:BAG_UPDATE()
	needsUpdate = true
end
function addon:PLAYER_LOGIN()
	if reagentSets[playerClass] then
		track = reagentSets[playerClass]
	else
		track = {}
	end
	
	if collapsingSets[playerClass] then
		trackAsOne = collapsingSets[playerClass]
	else
		trackAsOne = {}
	end

	table.insert(trackAsOne, healingPotions)
	if UnitPower("player", SPELL_POWER_MANA) > 0 then
		table.insert(trackAsOne, manaPotions)
	end

	local ammoItem = GetInventoryItemLink("player", 0)
	if ammoItem then
		table.insert(track, ammoItem)
	end

	for i = 0, 8 do -- Yes, I know it should be 0-7.
		qualityColor[#qualityColor + 1] = select(4, GetItemQualityColor(i))
	end

	self:RegisterEvent("BAG_UPDATE")
end
function addon:ADDON_LOADED(mod)
	if mod ~= "Broker_Reagents" then return end
	self:UnregisterEvent("ADDON_LOADED")
	BrokerReagentDBPC = BrokerReagentDBPC or {}
	db = BrokerReagentDBPC
end

do
	local total = 0
	addon:SetScript("OnUpdate", function(self, elapsed)
		total = total + elapsed
		if total > 5 then
			if needsUpdate then update() end
			total = 0
		end
	end)
end
addon:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)
addon:RegisterEvent("PLAYER_LOGIN")
addon:RegisterEvent("ADDON_LOADED")

do
	local nameFormat = "|T%s:16|t %s%s|r"
	local countFormat = "|c00%s%d|r"
	function bReagents.OnTooltipShow(tt)
		local added = nil
		for i, v in next, reagentNames do
			if not db[v] and reagentCount[v] >= max[v] then
				local n = nameFormat:format(icons[v], colors[v], v)
				local c = countFormat:format("00ff00", reagentCount[v])
				tt:AddDoubleLine(n, c)
				added = true
			end
		end
		for i, v in next, reagentNames do
			if not db[v] and reagentCount[v] < max[v] then
				if added then
					tt:AddLine(" ")
					added = nil
				end
				local n = nameFormat:format(icons[v], colors[v], v)
				local c = countFormat:format("ff0000", reagentCount[v])
				tt:AddDoubleLine(n, c)
			end
		end
	end
end

